/************************************************************
*        PROGRAMA DE INICIAO CIENTFICA PIBIC/CNPq        *
*   "Desenvolvimento de uma arquitetura para simulao do   *
*      Funcionamento distribudo e paralelo do crebro"     *
* --------------------------------------------------------- *
*      BOLSISTA: Anderson de Rezende Rocha - DCC/UFLA       *
*       ORIENTADOR: Antnio Maria Pereira de Resende        *
************************************************************/

import java.awt.*;
import javax.swing.*;

// Esta  a rea de desenho do programa. Todas
// as deliberaes a nvel de inferncias so interpretadas
// pelas outras classes e apenas representadas por esta 
// classe na tela de modo que o usurio acompanhe as atividades
// dos robots
public class AreaGrafica extends JPanel
{
   private long tempoEspera;
   private ImageIcon imagens[];
   private final int size = 8;
   private  final int deltaX = 60, deltaY = 60; 
   private int pontoX = -1, pontoY = -1, pontoX1 = -1, pontoY1 = -1;
   private char face;
   private boolean trava, pintarLimpeza, pintarVisitado, pintarObstaculo;

   Graphics g1;
   // Interessante para saber o tamanho til da tela de desenho 
   // disponvel. Nesse nvel j  discontado as bordas e o tamanho
   // dos menus
   private int areaX, areaY;

   public AreaGrafica()
   {
      super();
      setTempoEspera(0);
      criarImagens();
      trava = false;
      pintarLimpeza = false;
      pintarVisitado = false;
      pintarObstaculo = false;
   }

   // Criando o vetor de bots
   private void criarImagens()
   {
      imagens = new ImageIcon[size];

      for (int i = 0; i < size; i++)
         imagens[i] = new ImageIcon("img" + i + ".gif");
   }

   // Pinta os componentes a primeira vez
   public void paintComponent(Graphics g)
   {
      super.paintComponent(g);
      g1 = getGraphics();
   }

   public void pintar(Point pos, char face)
   {
      delay(tempoEspera);
      pontoX = (int)(deltaX * pos.getX());
      pontoY = (int)(deltaY * pos.getY());
      this.face = face;
      repaint();
   }

   public void limpar(Point pos)
   {
      pontoX1 = (int)(deltaX * pos.getX());
      pontoY1 = (int)(deltaY * pos.getY());

      trava = true;
      repaint();
      trava = false;
   }

   public void pintarLimpeza(Point pos)
   {
      pontoX1 = (int)(deltaX * pos.getX());
      pontoY1 = (int)(deltaY * pos.getY());

      pintarLimpeza = true;
      repaint();
      pintarLimpeza = false;
   }

   // Para a proposio atual no  necessrio
   // Futuramente pode vir a ser utilizado
   public void pintarVisitado(Point pos)
   {
      pontoX1 = (int)(deltaX * pos.getX());
      pontoY1 = (int)(deltaY * pos.getY());

      pintarVisitado = true;
      repaint();
      pintarVisitado = false;
   }

   public void pintarObstaculo(Point pos)
   {
      pontoX1 = (int)(deltaX * pos.getX());
      pontoY1 = (int)(deltaY * pos.getY());

      pintarObstaculo = true;
      repaint();
      pintarObstaculo = false;
   }
   
   // Funo para repintar
   public void repaint()
   {
      // Guardando os limites disponveis para desenho
      if (getWidth() != 0 && getHeight() != 0)
      {
         areaX = getWidth();
         areaY = getHeight();
      }

      int indice;

      if (g1 != null)
      {
         switch (face)
         {
            case 'n':
               indice = 1;
               break;
            case 's':
               indice = 0;
               break;
            case 'l':
               indice = 2;
               break;
            case 'o':
               indice = 3;
               break;
            default :
               indice = 0;
               break;
         }              
         if (trava)
            imagens[5].paintIcon(this, g1, pontoX1, pontoY1);         
         else 
            imagens[indice].paintIcon(this, g1, pontoX, pontoY);

         if (pintarLimpeza)
            imagens[6].paintIcon(this, g1, pontoX1, pontoY1);

         if (pintarVisitado)
            imagens[7].paintIcon(this, g1, pontoX1, pontoY1);

         if (pintarObstaculo)
            imagens[4].paintIcon(this, g1, pontoX1, pontoY1);
      }
      delay(tempoEspera);
   }

   // Tempo de inrcia.  escolhido pelo usurio 
   // na configurao do ambiente
   private void delay(long intervalo)
   {
      if (intervalo==0) return;

      long tempoInicial = System.currentTimeMillis();
      do 
      {
         // No Faz nada, apenas mata tempo.
         // cada intervalo de 1000 representa 1 segundo
      } while ( (System.currentTimeMillis()-tempoInicial) <= intervalo );
      
      return;
   }

   // Funo chamada para configurar o tempo de inrcia
   // do ambiente
   public void setTempoEspera(long tempo)
   {
      tempoEspera = tempo;
   }

   public void limparTudo()
   {
      pontoX = pontoY = pontoX1 = pontoY1 = 0;
      trava = pintarLimpeza = pintarVisitado = pintarObstaculo = false;

      int x = 0, y = 0;

      for (int i = 0; i < 7; i++)
      {
         x = i * 60;
         for (int j = 0; j < 7; j++ )
         {
            y = j * 60;
            imagens[5].paintIcon(this, g1, x, y);
         }
      }
   }
}