/************************************************************
*        PROGRAMA DE INICIAO CIENTFICA PIBIC/CNPq        *
*   "Desenvolvimento de uma arquitetura para simulao do   *
*      Funcionamento distribudo e paralelo do crebro"     *
* --------------------------------------------------------- *
*      BOLSISTA: Anderson de Rezende Rocha - DCC/UFLA       *
*       ORIENTADOR: Antnio Maria Pereira de Resende        *
************************************************************/

// Este  o robot propriamente dito. Pode ser instanciado sem        
// problemas. Cada instncia desta classe gera um robot independente 
// mas com o conhecimento idntico ao de seus irmos.                

// Importando o conjunto de pacotes do JESS
import jess.*;
// Importando a classe POINT para representao de dados
import java.awt.Point;
// Importando classe VECTOR para fazer retrospectivas de sujeiras
// e posies j visitadas
import java.util.Vector;
import java.lang.*;

// Definio do ROBOT
public class Robot extends Thread
{
   Rete robot;
   // Ponto [x,y] inicial 
   private Point posicaoAtual = new Point();
   private String face = new String();
   private AgenteCentralizador agente;
   public int robotID;
   private long timeWait;

   public boolean ativado = false;

   // Muda a direo do robot
   public void virar(String direcao)
   {
      if (direcao.equalsIgnoreCase(""))
         face = "norte";
      else if (!(face.equalsIgnoreCase(direcao)))
         face = direcao;

      agente.setFace(direcao.charAt(0), getID());
   }
   
   // Dado um ponto o robot anda para ele. No evita o tele-transporte
   // A passagem do ponto de destino no  verificada pois na base de 
   // conhecimento do robot est explcito que ele no tem a capacidade de se
   // teletransportar
   public void andar(Point destino)
   {
      // Andando uma posicao
      // Se posicao para andar for diferente da posicao 
      // atual ento robot est realmente andando
      if (!(posicaoAtual.equals(destino)))
      {
         posicaoAtual.setLocation(destino);         
         agente.setPosicao(posicaoAtual, getID());
      }
   }

   public int getID()
   {
      return robotID;
   }

   public void limpar(Point pto)
   {
      agente.limpar(pto, getID());
   }

   public void setEstadoInicial(Point pos, String face)
   {
      try
      {
         int x = (int)pos.getX();
         int y = (int)pos.getY();
         
         String comando = "(store posicao-y " + y + ")";
         robot.executeCommand(comando);
         comando = "(store posicao-x " + x + ")";
         robot.executeCommand(comando);
         comando = "(store face " + face + ")";
         robot.executeCommand(comando);

         comando = "(store obs-x -1)";
         robot.executeCommand(comando);

         comando = "(store obs-y -1)";
         robot.executeCommand(comando);

         comando = "(assert (posicao-atual(px " + x + ")(py " + y + ")))";
         robot.executeCommand(comando);

         comando = "(assert (direcao(face " + face + ")))";
         robot.executeCommand(comando);

         comando = "(assert (limites(norte 6)(sul 0)(leste 6)(oeste 0)))";
         robot.executeCommand(comando);

         comando = "(assert (contador(valor 0)))";
         robot.executeCommand(comando);

         comando = "(assert (obstaculo (sx -1)(sy -1)))";
         robot.executeCommand(comando);

         comando = "(assert (sortear-outra-vez))";
         robot.executeCommand(comando); 

         Point obstaculo = new Point();
         for (int i = 0; i < agente.obstaculos.size(); i++ )
         {
            obstaculo.setLocation((Point)(agente.obstaculos.elementAt(i))); 
            x = (int)obstaculo.getX();
            y = (int)obstaculo.getY();
            comando = "(assert (obstaculo(sx " + x + ")(sy " + y + ")))";
            robot.executeCommand(comando);
         }
      }
      catch (JessException jex)
      {
         System.err.println("Erro inesperado");
      }
   }
   
   // Construtor default do robot. Cria robots, configurando-os
   // com as caractersticas inerentes iniciais
   public Robot( AgenteCentralizador ag, String nome, int id, long tempo )
   {
      super(nome);
      try
      {
         this.agente = ag;
         robotID = id;
         // Carregando a base de conhecimentos do robot e
         // as circunstncias do ambiente
         this.carregarBaseRobot();
         // Adicionando um listener ao robot
         this.adicionarListener();
         timeWait = tempo;
         // Ligando o parmetro passado a uma varivel que seja global
      }
      catch (JessException ex)
      {
         System.err.println("Desculpe, ocorreu um erro inesperado!");
         // exibir o erro na tela para informacao do usuario
         System.err.println(ex);
      }
   }

   public void run()
   {
      System.out.println("\n[-> Robot" + getID() + " trabalhando]\n");
      try
      {
      	Thread.sleep(timeWait);
      }
      catch (InterruptedException ie)
      {
         System.err.println("Erro inesperado!");
      }

      try
      {
         iniciarRobot();      	
      }
      catch (JessException jex)
      {
         System.err.println();
      }
   }

   // Coloca a inteligncia no robot
   private void carregarBaseRobot() throws JessException
   {
      // criacao do objeto RETE
      robot = new Rete();      
      robot.executeCommand("(batch templates.clp)");
      robot.executeCommand("(batch conhecimento.clp)");
      robot.start();
   }

   // Adiciona um ouvinte s aes do robot
   private void adicionarListener()
   {
      // Declarando os listeners
      robot.addJessListener(new HandlerRobot(this));
      robot.setEventMask(robot.getEventMask() | JessEvent.FACT | JessEvent.HALT | JessEvent.RUN);
   }

   // Responsvel pelo incio das atividades prescritas 
   // na base de inteligncia do robot
   public void iniciarRobot() throws JessException
   {
      System.out.println("[--------- Robot " + getID() + " iniciando execucao]");
      robot.Run();
      System.out.println("[Robot" + getID() + " terminando deliberacao");
   }
}