/************************************************************
*        PROGRAMA DE INICIAO CIENTFICA PIBIC/CNPq        *
*   "Desenvolvimento de uma arquitetura para simulao do   *
*      Funcionamento distribudo e paralelo do crebro"     *
* --------------------------------------------------------- *
*      BOLSISTA: Anderson de Rezende Rocha - DCC/UFLA       *
*       ORIENTADOR: Antnio Maria Pereira de Resende        *
************************************************************/

import java.awt.*;
import javax.swing.*;

public class Painel extends JPanel
{
   private ImageIcon iconeRobots[];
   private int pontoX = 0, pontoY = 0;
   private int sujeiraX = 0, sujeiraY = 0;
   private int visitadoX = 0, visitadoY = 0;
   private int limpoX = 0, limpoY = 0;
   private int deltaX = 60, deltaY = 60;
   private boolean desenharSujeira, desenharVisitado, precisaLimpar;
   private char face;
   private long tempoEspera;

   Graphics g1;

   public Painel()
   {
      super();
      face = 's';
      criarIcones();
      precisaLimpar = false;
   }

   private void criarIcones()
   {
      iconeRobots = new ImageIcon[8];

      for (int i = 0; i < iconeRobots.length; i++)
      {
         iconeRobots[i] = new ImageIcon("robot" + i + ".GIF");
      }
   }
   
   public void paintComponent(Graphics g)
   {
      super.paintComponent(g);
      g1 = getGraphics();

      iconeRobots[1].paintIcon(this, g, pontoX, pontoY);
   }

   public void draw(int x, int y)
   {
      precisaLimpar = true;
      repaint();
      precisaLimpar = false;

      pontoX = (deltaX * x);
      pontoY = (deltaY * y);

      repaint();
      delay(tempoEspera);
   }

   public void repaint()
   {
      int indice;

      if (precisaLimpar)
         iconeRobots[4].paintIcon(this, g1, pontoX, pontoY);
      else if (g1 != null)
      {
         switch (face)
         {
            case 'n':
               indice = 0;
               break;
            case 's':
               indice = 1;
               break;
            case 'l':
               indice = 2;
               break;
            case 'o':
               indice = 3;
               break;
            default :
               indice = 0;
               break;
         }              
         iconeRobots[indice].paintIcon(this, g1, pontoX, pontoY);
         iconeRobots[6].paintIcon(this, g1, limpoX, limpoY);

         if (desenharSujeira)
         {
            iconeRobots[5].paintIcon(this, g1, sujeiraX, sujeiraY);
            desenharSujeira = false;
         }
      }
   }

   private void delay(long intervalo)
   {
      if (intervalo==0) return;

      long tempoInicial = System.currentTimeMillis();
      do 
      {
         // No Faz nada, apenas mata tempo.
         // cada intervalo de 1000 representa 1 segundo
      } while ( (System.currentTimeMillis()-tempoInicial) <= intervalo );
      
      return;
   }

   public void setFace(char fc)
   {
      face = fc;
   }

   public void exibirVisitado(Point pto)
   {
      visitadoX = (deltaX * (int)pto.getX());
      visitadoY = (deltaX * (int)pto.getY());
      desenharVisitado = true;
      repaint();
   }

   public void limparPto(Point pto)
   {
      limpoX = (deltaX * (int)pto.getX());
      limpoY = (deltaY * (int)pto.getY());
      repaint();
   }

   public void exibirSujeira(Point suj)
   {
      sujeiraX = (deltaX * (int)suj.getX());
      sujeiraY = (deltaY * (int)suj.getY());
      desenharSujeira = true;
      repaint();
   }

   public void limpar()
   {
      pontoX = 0;
      pontoY = 0;
      sujeiraX = 0;
      sujeiraX = 0;
      limpoX = 0;
      limpoY = 0;
      desenharSujeira = desenharVisitado = precisaLimpar = false;

      int x = 0, y = 0;

      for (int i = 0; i < 7; i++)
      {
         x = i * 60;
         for (int j = 0; j < 7; j++ )
         {
            y = j * 60;
            iconeRobots[7].paintIcon(this, g1, x, y);
         }
      }
      iconeRobots[1].paintIcon(this, g1, 0, 0);
   }

   public void setTempoEspera(long tempo)
   {
      tempoEspera = tempo;
   }
}