/************************************************************
*        PROGRAMA DE INICIAO CIENTFICA PIBIC/CNPq        *
*   "Desenvolvimento de uma arquitetura para simulao do   *
*      Funcionamento distribudo e paralelo do crebro"     *
* --------------------------------------------------------- *
*      BOLSISTA: Anderson de Rezende Rocha - DCC/UFLA       *
*       ORIENTADOR: Antnio Maria Pereira de Resende        *
************************************************************/

// Este  o robot propriamente dito. Pode ser instanciado sem        
// problemas. Cada instncia desta classe gera um robot independente 
// mas com o conhecimento idntico ao de seus irmos.                

// Importando o conjunto de pacotes do JESS
import jess.*;
// Importando a classe POINT para representao de dados
import java.awt.Point;
// Importando classe VECTOR para fazer retrospectivas de sujeiras
// e posies j visitadas
import java.util.Vector;
import java.lang.*;

// Definio do ROBOT
public class Robot
{
   Rete robot;
   // Ponto [x,y] inicial criado automaticamente em [0,0]
   private Point posicaoAtual = new Point(0,0);
   // Um posio j visitada
   private Point visitado = new Point(0,0);
   // Retrospectiva dos locais j visitados pelo robot
   private Vector jaVisitados = new Vector();
   // Locais onde h configurao de sujeira
   private Vector sujeira = new Vector();
   // Face do robot aponta para norte, sul, leste ou oeste
   private String face = new String();
   // Guardar a referencia  interface grfica
   private GUIBot gui;
   private KqmlCliente cliente = new KqmlCliente("127.0.0.1",5000);

   public boolean ativado = false;

   // setFace  privado apenas por enquanto
   private void setFace(String direcao)
   {
      // Por default direcao  norte. Robot s vira se direcao 
      // passada for difente da direcao (face) atual
      if (direcao.equalsIgnoreCase(""))
         face = "norte";
      else if (!(face.equalsIgnoreCase(direcao)))
         face = direcao;
   }

   // Retorna a posicao do robot atualmente
   public Point getPosicao()
   {
      return posicaoAtual;
   }

   // Retorna uma string do tipo "norte" se o robot estiver
   // virado para norte
   public String getFace()
   {
      return face;
   }

   // Retorna uma posio visitada
   public Point getVisitado()
   {
      return visitado;
   }
   
   // Seleciona uma posio como j visitado e como j limpo
   public void setLimpeza(Point pto)
   {
      gui.exibirLimpeza(pto);
   }
   
   // Seleciona uma posio como visitada
   public void setVisitado(Point pto)
   {
      // Se a casa ainda no foi visitada...
      if (jaVisitados.indexOf(pto) != -1)
      {
         jaVisitados.addElement(pto);

         String comando = "setVisitado("+(int)pto.getX()+","+(int)pto.getY()+")";
         cliente.enviarMensagem(comando, "HALT");
         cliente.runClient();        
      }
   }

   // Muda a direo do robot
   public void virar(String direcao)
   {
      setFace(direcao);
   }
   
   // Dado um ponto o robot anda para ele. No evita o tele-transporte
   // A passagem do ponto de destino no  verificada pois na base de 
   // conhecimento do robot est explcito que ele no tem a capacidade de se
   // teletransportar
   public void andar(Point destino)
   {
      // Andando uma posicao
      // Se posicao para andar for diferente da posicao 
      // atual ento robot est realmente andando
      if (!(posicaoAtual.equals(destino)))
      {
         posicaoAtual.setLocation(destino);         

        String comando = "setEstado("+(int)getPosicao().getX()+","+(int)getPosicao().getY()+");"+
            "setFace(" + getFace() + ")";
        cliente.enviarMensagem(comando, "HALT");
        cliente.runClient();        
      }
   }

   // Construtor default do robot. Cria robots, configurando-os
   // com as caractersticas inerentes iniciais
   public Robot( GUIBot gui )
   {
      try
      {
         // Carregando a base de conhecimentos do robot e
         // as circunstncias do ambiente
         this.carregarBaseRobot();
         // Adicionando um listener ao robot
         this.adicionarListener();
         this.setFace("norte");
         // Ligando o parmetro passado a uma varivel que seja global
         this.gui = gui;
      }
      catch (JessException ex)
      {
         System.err.println("Desculpe, ocorreu um erro inesperado!");
         // exibir o erro na tela para informacao do usuario
         System.err.println(ex);
      }
   }

   // Coloca a inteligncia no robot
   private void carregarBaseRobot() throws JessException
   {
      // criacao do objeto RETE
      robot = new Rete();      
      // carregando o arquivo de templates
      robot.executeCommand("(batch defTemplates.clp)");
      // carregando a base de regras
      robot.executeCommand("(batch rules.clp)");
      // carregando os estados iniciais
      robot.executeCommand("(batch factsEstadoInicial.clp)");
   }

   // Adiciona um ouvinte s aes do robot
   private void adicionarListener()
   {
      // Declarando os listeners
      robot.addJessListener(new HandlerRobot(this));
      robot.setEventMask(robot.getEventMask() | JessEvent.FACT | JessEvent.HALT | JessEvent.RUN);
   }

   // Responsvel pelo incio das atividades prescritas 
   // na base de inteligncia do robot
   public void iniciarRobot() throws JessException
   {
      // Carregando os limites do local
      Point ptoSujeira;

      // Asserindo o limite da matriz de ambiente
      String comando = "(assert (limites(norte 6)(sul 0)(oeste 0)(leste 6)))";

      // Executando a assero acima
      robot.executeCommand(comando);

      // Ensinando  base do robot onde h obstculos. Obstculos que
      // foram previamente selecionados pelo usurio
      for (int i = 0; i < gui.sujeiras.size(); i++)
      {
         comando = "";
         ptoSujeira = (Point)gui.sujeiras.elementAt(i);
         comando = "(assert (sujeira(sx " + (int)ptoSujeira.getX() + 
                   ")(sy " + (int)ptoSujeira.getY() + ")))";
         
         robot.executeCommand(comando);
         comando = "(store sx " + (int)ptoSujeira.getX() + ")";
         robot.executeCommand(comando);
         comando = "(store sy " + (int)ptoSujeira.getY() + ")";
         robot.executeCommand(comando);
      }

      // Depois de dizer  base de conhecimentos do robot onde esto os
      // obstculos deve-se avisar  gui para exibir na tela onde esto
      // estes obstculos
      gui.exibirSujeiras();
      // Dando incio ao motor de inferncia do robot
      robot.run();
   }
}