/************************************************************
*        PROGRAMA DE INICIAO CIENTFICA PIBIC/CNPq        *
*   "Desenvolvimento de uma arquitetura para simulao do   *
*      Funcionamento distribudo e paralelo do crebro"     *
* --------------------------------------------------------- *
*      BOLSISTA: Anderson de Rezende Rocha - DCC/UFLA       *
*       ORIENTADOR: Antnio Maria Pereira de Resende        *
************************************************************/

// Este  o robot propriamente dito. Pode ser instanciado sem        
// problemas. Cada instncia desta classe gera um robot independente 
// mas com o conhecimento idntico ao de seus irmos.                

// Importando o conjunto de pacotes do JESS
import jess.*;
// Importando a classe POINT para representao de dados
import java.awt.Point;
// Importando classe VECTOR para fazer retrospectivas de sujeiras
// e posies j visitadas
import java.util.Vector;
import java.lang.*;

import java.rmi.*;
import java.rmi.server.*;


// Definio do ROBOT
public class RobotImpl extends UnicastRemoteObject 
                   implements Runnable, Robot
{
   final int LIMITESGUI = 7;
   private Rete rete;
   Thread th;
   // Ponto [x,y] inicial 
   private Point posicaoAtual = new Point();
   private String face = new String();
   private HandlerRobot handler;
   private String localHost, remoteHost;
   public String robotID;
   private Vector botsCompanheiros = new Vector();
   private Vector IDsBotsCompanheiros = new Vector();
   private Vector obstaculos = new Vector();
   private Vector jaVisitados = new Vector();

   // Muda a direo do robot
   public void setFace(String direcao)
   {
      if (direcao.equalsIgnoreCase(""))
         face = "norte";
      else if (!(face.equalsIgnoreCase(direcao)))
         face = direcao;

      try
      {
      	GGrafRobot gGraf = requestGraf();
         gGraf.desenharPosicao(posicaoAtual, face.charAt(0));
      }
      catch (RemoteException re)
      {
         System.err.println(re);
      }
   }
   
   public void limpar(Point destino)
   {
      try
      {
      	GGrafRobot gGraf = requestGraf();
         gGraf.desenharLimpeza(destino);
      }
      catch (RemoteException re)
      {
         System.err.println(re);
      }

   }

   public String getID()
   {
      return robotID;
   }

   public String getIP()
   {
      return localHost;
   }

   private void setEstadoInicial()
   {
      int x = (int)posicaoAtual.getX();
      int y = (int)posicaoAtual.getY();
      int sorteio = 1 + (int)(Math.random() * 4);
      Point obstaculo = new Point();
      String face, comando;
          
      switch (sorteio)
      {
         case 1:
            face = "norte";
            break;
         case 2:
            face = "sul";
            break;
         case 3:
            face = "leste";
            break;
         case 4:
            face = "oeste";
            break;
         default:
            face = "norte";
            break;
      }      

      try
      {
         comando = "(store posicao-y " + y + ")";
         rete.executeCommand(comando);
         comando = "(store posicao-x " + x + ")";
         rete.executeCommand(comando);
         comando = "(store face " + face + ")";
         rete.executeCommand(comando);
         comando = "(store obs-x -1)";
         rete.executeCommand(comando);
         comando = "(store obs-y -1)";
         rete.executeCommand(comando);
         comando = "(store vis-x -1)";
         rete.executeCommand(comando);
         comando = "(store vis-y -1)";
         rete.executeCommand(comando);
         comando = "(store verificar-possibilidades 1)";
         rete.executeCommand(comando);

         comando = "(assert (posicao-atual(px " + x + ")(py " + y + ")))";
         rete.executeCommand(comando);

         comando = "(assert (direcao(face " + face + ")))";
         rete.executeCommand(comando);

         comando = "(assert (limites(norte " + (LIMITESGUI-1) + ")(sul 0)(leste " 
                   + (LIMITESGUI-1) + ")(oeste 0)))";
         rete.executeCommand(comando);

         comando = "(assert (configurar-estado-inicial))";
         rete.executeCommand(comando);

         for (int i = 0; i < obstaculos.size(); i++)
         {
            obstaculo.setLocation((Point)(obstaculos.elementAt(i)));
            x = (int)obstaculo.getX();
            y = (int)obstaculo.getY();
            comando = "(store obs-x " + x + ")";
            rete.executeCommand(comando);
            comando = "(store obs-y " + y + ")";
            rete.executeCommand(comando);
            comando = "(assert (obstaculo (sx " + x + ")(sy " + y + ")))";
            rete.executeCommand(comando);
         }   
      }
      catch (JessException jex)
      {
         System.err.println("Erro inesperado (Excecao em RobotImpl.java)");
         System.err.println(jex);
      }
   }
   
   // Construtor default do robot. Cria robots, configurando-os
   // com as caractersticas inerentes iniciais
   public RobotImpl(String ipLocal, String ipRemoto, String xInicial, String yInicial, String id) throws Exception
   {
      try
      {
         localHost = ipLocal;
         remoteHost = ipRemoto;
         int x = Integer.parseInt(xInicial);
         int y = Integer.parseInt(yInicial);
         posicaoAtual.setLocation(x,y);         
         robotID = "Robot-" + id;

         // Carregando a base de conhecimentos do robot e
         // as circunstncias do ambiente
         this.carregarBaseRobot();
         // Adicionando um listener ao robot
         this.adicionarListener();
      }
      catch (JessException ex)
      {
         System.err.println("Desculpe, ocorreu um erro inesperado!");
         // exibir o erro na tela para informacao do usuario
         System.err.println(ex);
      }
   }

   public void run()
   {
      System.out.println("\n[-> " + getID() + " trabalhando]\n");
      try
      {
         iniciarRobot();      	
      }
      catch (JessException jex)
      {
         System.err.println(jex);
      }
   }

   public void start()
   {
      th = new Thread(this, getID());
      setPosicao(posicaoAtual);      	
      run();
   }

   public void setPosicao(Point posicao)
   {
      try
      {
         // Fazendo requisio ao servidor grfico
         GGrafRobot gGraf = requestGraf();
         posicaoAtual.setLocation(posicao);   
         gGraf.desenharPosicao(posicaoAtual, face.charAt(0));
      }
      catch (RemoteException re)
      {
         System.err.println(re);
      }
   }

   public void sendVisitado(Point pto)
   {
      if (!jaVisitados.contains(pto))
         jaVisitados.addElement(pto);
   }
   
   public synchronized void setVisitado(Point pto)
   {
      try
      {
         // Avisando os outros
         avisarCompanheiros(pto);
         GGrafRobot gGraf = requestGraf();
         gGraf.desenharVisitado(pto);               
         if (!jaVisitados.contains(pto))
            jaVisitados.addElement(pto);
      }
      catch (RemoteException re)
      {
         System.err.println(re);
      }
   }


   // Iterao que avisa aos outros robots que uma certa posio j foi visitada
   public void avisarCompanheiros(Point pto) throws RemoteException
   {
         // Parte que avisa aos outros robots sobre os visitados
         for (int i = 0; i < botsCompanheiros.size(); i++)
         {
            Robot bot = requestRobot((String)botsCompanheiros.elementAt(i), 
                        (String)IDsBotsCompanheiros.elementAt(i));

            bot.sendVisitado(pto);
         }
   }
   
   public int verificarSensorObstaculos(Point pto)   
   {
      if (obstaculos.contains(pto))
         return 1;
      return 0;      
   }

   private boolean existeVisitado(Point pto)
   {
      if (jaVisitados.contains(pto))
         return true;
      return false;      
   }

   
   public void verificarPossibilidades(Point pto)
   {
      int x = (int)(pto.getX());
      int y = (int)(pto.getY());
      int possibilidades[] = new int[4];
      int sensor[] = new int[4];
      Point aux = new Point();
      String comando = "", comando1 = "";

      for (int i = 0; i < 4; i++)
      {
         switch (i)
         {
            case 0:
               if (y+1 < LIMITESGUI)
               {
                  aux.setLocation(x, y+1);                  
                  if (existeVisitado(aux))
                     possibilidades[i] = 0;
                  else
                  {
                     possibilidades[i] = 1;
                     sensor[i] = verificarSensorObstaculos(aux);
                  }
               }
               else
                  possibilidades[i] = 0;
               break;
            case 1:
               if (y-1 >= 0)
               {
                  aux.setLocation(x, y-1);                  
                  if (existeVisitado(aux))
                     possibilidades[i] = 0;   
                  else
                  {
                     possibilidades[i] = 1;   
                     sensor[i] = verificarSensorObstaculos(aux);
                  }
               }
               else
                  possibilidades[i] = 0;
               break;
            case 2:
               if (x+1 < LIMITESGUI)
               {
                  aux.setLocation(x+1, y);                  
                  if (existeVisitado(aux))
                     possibilidades[i] = 0;   
                  else
                  {
                     possibilidades[i] = 1;   
                     sensor[i] = verificarSensorObstaculos(aux);
                  }
               }
               else
                  possibilidades[i] = 0;
               break;
            default:
               if (x-1 >= 0)
               {
                  aux.setLocation(x-1, y);                  
                  if (existeVisitado(aux))
                     possibilidades[i] = 0;   
                  else
                  {
                     possibilidades[i] = 1;   
                     sensor[i] = verificarSensorObstaculos(aux);
                  }                   
               }
               else
                  possibilidades[i] = 0;
               break;
         }
      }

      try
      {
         int soma = 0;
         for (int i = 0; i < 4; i++)
         {
            soma += sensor[i];
         }
       if (soma != 0)
       {
          rete.executeCommand("(sorteiar " + 
               sensor[0] + " " + sensor[1] + " " +
               sensor[2] + " " + sensor[3] + ")");           
       }
       else
       {
            rete.executeCommand("(sorteiar " + 
                 possibilidades[0] + " " + possibilidades[1] + " " +
                 possibilidades[2] + " " + possibilidades[3] + ")");           
       }

         Value tmp = rete.fetch("retorno");
         int valor = tmp.intValue(rete.getGlobalContext());
         int px, py;
         Point novaPosicao;
         switch (valor)
         {
            case 1:
               py = y + 1;
               novaPosicao = new Point(x, py);
               setVisitado(novaPosicao);
               break;
            case 2:
               py = y - 1;
               novaPosicao = new Point(x, py);
               setVisitado(novaPosicao);
               break;
            case 3:
               px = x + 1;
               novaPosicao = new Point(px, y);
               setVisitado(novaPosicao);
               break;
            case 4:
               px = x - 1;
               novaPosicao = new Point(px, y);
               setVisitado(novaPosicao);
               break;
         }   
      }
      catch (JessException jex)
      {
         System.err.println(jex);
      }
   }

   public void terminarRobot()
   {  
      try
      {
      	GGrafRobot gGraf = requestGraf();
         gGraf.terminarEsteBot(posicaoAtual, face.charAt(0), getID(), getIP());
         // Ativar em caso de destruio da THREAD atual
         // rete.destroy();
         // th.destroy();
      }
      catch (RemoteException re)
      {
         System.err.println(re);
      }
      // Mostrar ao gGraf o termino deste robot 
      // e matar a thread
      // gGraf.terminarEsteRobot(getID());
   }
   
   // Coloca a inteligncia no robot
   private void carregarBaseRobot() throws JessException
   {
      // criacao do objeto RETE
      rete = new Rete();      
      rete.executeCommand("(batch template.clp)");
      rete.executeCommand("(batch conhecimento.clp)");
      rete.start();
   }

   // Adiciona um ouvinte s aes do robot
   private void adicionarListener()
   {
      // Declarando os listeners
      this.handler = new HandlerRobot(this);
      rete.addJessListener(this.handler);
      rete.setEventMask(rete.getEventMask() | JessEvent.FACT | JessEvent.HALT | JessEvent.RUN);
   }

   // Responsvel pelo incio das atividades prescritas 
   // na base de inteligncia do robot
   public void iniciarRobot() throws JessException
   {
      System.out.println("[--------- Robot" + getID() + " comecou a execucao]");
      rete.Run();
      System.out.println("[" + getID() + " terminou execucao");
   }

   public static void main(String args[]) throws Exception
   {
      RobotImpl instancia = new RobotImpl(args[0], args[1], args[2], args[3], args[4]);
      instancia.registrarIP(instancia);
      instancia.setEstadoInicial();
      // Bastar acionar esta linha para o robot entrar em execuo
      instancia.start();
   }

   public void registrarIP(RobotImpl instancia)
   {
      try 
      {
         Naming.rebind("//"+ localHost + "/" + getID(), instancia);
         System.out.println("Estou registrado (" + getID() + ")");
         apresentarSeGGraf();
      } 
      catch (Exception ex) 
      {
         System.err.println("Sistema remoto nao encontrado. Tentando conectar novamente");
      }
   }

   public void apresentarSeGGraf() 
   {
      boolean contato = false;

      while (!contato)
      {
         try 
         {
            GGrafRobot gGraf = (GGrafRobot)(Naming.lookup("//"+remoteHost+"/GGrafRobot"));
            System.out.println("Contato foi um sucesso");
            contato = true;
            updateCompanheiros(gGraf.apresentacao(localHost, getID()), gGraf.getIDs());
            setObstaculos(gGraf.getObstaculos());
         } 
         catch (Exception ex) 
         {
            contato = false;
            System.err.println("Contato nao estabelecido");
         }
      }
   }

   // Comunica-se com o gGraf
   private GGrafRobot requestGraf() throws RemoteException
   {
      boolean contato = false;
      GGrafRobot gGraf = null;

      while (!contato)
      {
         try 
         {
            gGraf = (GGrafRobot)(Naming.lookup("//"+remoteHost+"/GGrafRobot"));
            contato = true;
         } 
         catch (Exception ex) 
         {
            contato = false;
            System.err.println("Contato nao estabelecido");
         }
      }
      return gGraf;
   }

   // Comunica-se com um robot
   private Robot requestRobot(String ip, String id) throws RemoteException
   {
      boolean contato = false;
      Robot bot = null;

      while (!contato)
      {
         try 
         {
            bot = (Robot)(Naming.lookup("//"+ip+"/"+id));
            contato = true;
         } 
         catch (Exception ex) 
         {
            contato = false;
            System.err.println("Contato nao estabelecido");
         }
      }
      return bot;
   }

   public void inserirCompanheiro(String ip, String id) throws RemoteException
   {
      botsCompanheiros.addElement(ip);
      IDsBotsCompanheiros.addElement(id);
   }

   private void updateCompanheiros(Vector bots, Vector ids)
   {
      botsCompanheiros = bots;
      IDsBotsCompanheiros = ids;
      botsCompanheiros.removeElement(localHost);
      IDsBotsCompanheiros.removeElement(getID());
   }

   private void setObstaculos(Vector obst) throws JessException
   {
      obstaculos = obst;
      System.out.println(obstaculos);
   }
}